
# Load the SOAP service support framework
package require SOAP::Domain

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

#    The following are taken from /MIDAS/tcl/common.tcl
#    This file will not be loaded if TclHttpd is run outside of a MIDAS-session environment

proc first p {lindex $p 0}
proc second p {lindex $p 1}
proc third p {lindex $p 2}
proc fourth p {lindex $p 3}
proc fifth p {lindex $p 4}
proc sixth p {lindex $p 5}
proc seventh p {lindex $p 6}
proc eigth p {lindex $p 7}
proc eighth p {lindex $p 7}
proc nineth p {lindex $p 8}
proc ninth p {lindex $p 8}
proc tenth p {lindex $p 9}

proc last p {lindex $p [expr [llength $p]-1]}
proc front p {lreplace $p [expr [llength $p]-1] [expr [llength $p]-1]}
proc head p {lindex $p 0}
proc tail p {lreplace $p 0 0}


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer {optsname sock args} {
    global MIDAS_SOAPDomainHandler_Verbose
    variable debug
    upvar \#0 Httpd$sock data
    upvar \#0 $optsname options

  if {$MIDAS_SOAPDomainHandler_Verbose} {
      puts "::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer called with $optsname || $sock || $args"
      foreach member [array names options] {puts "options($member) =  $options($member)"}
      foreach member [array names data] {puts "data($member) =  $data($member)"}
      puts ""
  }

    # if suffix is {} then it fails to make it through the various evals.
    set suffix [lindex $args 0]
    
    # check this is an XML post
    set failed [catch {set type $data(mime,content-type)} msg]
    if { $failed } {
        set msg "Invalid SOAP request: not XML data"
        log::log debug $msg
        Httpd_ReturnData $sock text/xml [SOAP::fault SOAP-ENV:Client $msg] 500
        return $failed
    }
    
    # make sure we were sent some XML
    set failed [catch {set query $data(query)} msg]
    if { $failed } {
        set msg "Invalid SOAP request: no data sent"
        log::log debug $msg
        Httpd_ReturnData $sock text/xml [SOAP::fault SOAP-ENV:Client $msg] 500
        return $failed
    }

    # Check that we have a properly registered domain
    if { ! [info exists options] } {
        set msg "Internal server error: domain improperly registered"
        log::log debug $msg
        Httpd_ReturnData $sock text/xml [SOAP::fault SOAP-ENV:Server $msg] 500
        return 1
    }   

   set argValues {}

   set frags [split $data(query) >]
   set fragNum [llength $frags]

   for {set i 0} {$i < $fragNum} {incr i} {
       set frag [lindex $frags $i]
       if {[string compare -length 14 $frag <SOAP-ENV:Body] == 0} {
            incr i
            set frag [lindex $frags $i]
            set methodName [string range [lindex [split $frag " "] 0] 4 end]
            incr i
            for {} {$i < $fragNum} {incr i} {
                set frag [lindex $frags $i]
                if {[string compare -length 3 $frag </ns] == 0} {break}
                    incr i
                    set frag [lindex $frags $i]
                    lappend argValues [lindex [split $frag <] 0]
            }
       }
   }

  if {$MIDAS_SOAPDomainHandler_Verbose} {
      puts "evaluate the method:- interp eval  ::namespace inscope $options(-namespace) $methodName $argValues"
  }

       # evaluate the method
       set z [catch {interp eval "" ::namespace inscope $options(-namespace) $methodName $argValues} msg]

       set xml "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
       append xml "<SOAP-ENV:Envelope xmlns:xsi=\"http://www.w3.org/1999/XMLSchema-instance\" "
       append xml "xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" "
       append xml "xmlns:SOAP-ENC=\"http://schemas.xmlsoap.org/soap/encoding/\" "
       append xml "xmlns:xsd=\"http://www.w3.org/1999/XMLSchema\">"

       append xml "<SOAP-ENV:Body xmlns:ns=\"$options(-uri)\" SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"

    if {$z == 0}  {

       append xml "<ns:[set methodName]Response>"
       if {[lindex $msg 0] == "rpcvar1.2"} {
           append xml "<result xsi:type=\"xsd:[lindex $msg 4]\">[lindex $msg 5]</result>"
       } else {
           foreach item $msg {
               append xml "<[lindex $item 0] xsi:type=\"xsd:[lindex $item 1]\">[lindex $item 2]</[lindex $item 0]>"
           }
       }
       append xml "</ns:[set methodName]Response>"

       set rc 200

    } else {

       global errorCode errorInfo
    
       append xml "<SOAP-ENV:Fault>"
       append xml "<faultcode>SOAP-ENV:Server</faultcode>"
       append xml "<faultstring>Error detected in method $methodName</faultstring>"
       append xml "<detail>"
       append xml "<e:errordetails xmlns:e=\"urn:TclSOAP-ErrorInfo\">"
       append xml "<errorcode>$errorCode</errorcode>"
       append xml "<errorinfo>$errorInfo</errorinfo>"
       append xml "</e:errordetails>"
       append xml "</detail>"
       append xml "</SOAP-ENV:Fault>"

       set rc 500
      
    }

       append xml "</SOAP-ENV:Body>"

       append xml "</SOAP-ENV:Envelope>"

       set xml [encoding convertto utf-8 $xml]          ;# make it UTF-8

       Httpd_ReturnData $sock text/xml $xml $rc

  if {$MIDAS_SOAPDomainHandler_Verbose} {
      puts "response:- z=$z  msg=$msg\n xml=$xml"
  }

    return 0
}

# -------------------------------------------------------------------------



